# Copyright 2021 UW-IT, University of Washington
# SPDX-License-Identifier: Apache-2.0

from unittest import TestCase
from uw_sws.section import get_section_by_label
from uw_pws import PWS
from uw_sws.util import fdao_sws_override
from uw_pws.util import fdao_pws_override
from uw_catalyst.util import fdao_catalyst_override
from uw_catalyst.gradebook import *
from uw_catalyst.exceptions import InvalidGradebookID
import mock


@fdao_sws_override
@fdao_pws_override
@fdao_catalyst_override
class CatalystTestGradebook(TestCase):
    def test_invalid_gradebook_id(self):
        self.assertRaises(
            InvalidGradebookID, get_participants_for_gradebook, None)
        self.assertRaises(
            InvalidGradebookID, get_participants_for_gradebook, '')
        self.assertRaises(
            InvalidGradebookID, get_participants_for_gradebook, 'abc')
        self.assertRaises(
            InvalidGradebookID, get_gradebook_export, 00000)
        self.assertRaises(
            InvalidGradebookID, get_gradebook_export, 11111111111)

    @mock.patch('uw_catalyst.gradebook.get_resource')
    def test_gradebook_export(self, mock_fn):
        person = PWS().get_person_by_netid('bill')

        response = get_gradebook_export(12345, person)

        mock_fn.assert_called_with('/rest/gradebook/v1/book/12345/export', {
            'Accept': 'application/vnd.ms-excel',
            'X-UW-Act-as': 'bill'})

    def test_participants_for_gradebook(self):
        person = PWS().get_person_by_netid('bill')

        participants = get_participants_for_gradebook(12345, person)

        self.assertEquals(len(participants), 3, 'Correct participant count')

    def test_participants_for_section(self):
        section = get_section_by_label('2013,summer,CSS,161/A')
        instructor = section.meetings[0].instructors[0]

        participants = get_participants_for_section(section, instructor)

        self.assertEquals(len(participants), 3, 'Correct participant count')



# Copyright 2021 UW-IT, University of Washington
# SPDX-License-Identifier: Apache-2.0

"""
Interface for interacting with Catalyst WebQ Survey
"""

from uw_catalyst.exceptions import InvalidSurveyID
from uw_catalyst import get_resource
import re


def get_survey_export(survey_id, person=None):
    """
    Returns a survey exported as an .xls file
    """
    if not valid_survey_id(survey_id):
        raise InvalidSurveyID(survey_id)

    url = "/rest/webq/v2/survey/{}/export".format(survey_id)
    headers = {"Accept": "application/zip"}

    if person is not None:
        headers["X-UW-Act-as"] = person.uwnetid

    return get_resource(url, headers)


def get_survey_results(survey_id, person=None):
    """
    Returns survey result data as an .xls file
    """
    if not valid_survey_id(survey_id):
        raise InvalidSurveyID(survey_id)

    url = "/rest/webq/v2/survey/{}/responses".format(survey_id)
    headers = {"Accept": "application/vnd.ms-excel"}

    if person is not None:
        headers["X-UW-Act-as"] = person.uwnetid

    return get_resource(url, headers)


def get_survey_code_translation(survey_id, person=None):
    """
    Returns the survey code translation table as a .csv file
    """
    if not valid_survey_id(survey_id):
        raise InvalidSurveyID(survey_id)

    url = "/rest/webq/v2/survey/{}/code-translation".format(survey_id)
    headers = {"Accept": "text/csv"}

    if person is not None:
        headers["X-UW-Act-as"] = person.uwnetid

    return get_resource(url, headers)


def valid_survey_id(survey_id):
    if re.match(r"^[1-9][0-9]{,9}$", str(survey_id)):
        return True
    return False



# Copyright 2021 UW-IT, University of Washington
# SPDX-License-Identifier: Apache-2.0

from commonconf import override_settings


fdao_catalyst_override = override_settings(
    RESTCLIENTS_CATALYST_DAO_CLASS='Mock')



# Copyright 2021 UW-IT, University of Washington
# SPDX-License-Identifier: Apache-2.0

from unittest import TestCase
from uw_pws import PWS
from uw_pws.util import fdao_pws_override
from uw_catalyst.util import fdao_catalyst_override
from uw_catalyst.survey import *
from uw_catalyst.exceptions import InvalidSurveyID
import mock


@fdao_pws_override
@fdao_catalyst_override
class CatalystTestSurvey(TestCase):
    def test_invalid_survey_id(self):
        self.assertRaises(
            InvalidSurveyID, get_survey_results, None)
        self.assertRaises(
            InvalidSurveyID, get_survey_results, '')
        self.assertRaises(
            InvalidSurveyID, get_survey_export, 'abc')
        self.assertRaises(
            InvalidSurveyID, get_survey_export, 00000)
        self.assertRaises(
            InvalidSurveyID, get_survey_code_translation, 11111111111)

    @mock.patch('uw_catalyst.survey.get_resource')
    def test_survey_export(self, mock_fn):
        person = PWS().get_person_by_netid('bill')

        response = get_survey_export(12345, person)

        mock_fn.assert_called_with('/rest/webq/v2/survey/12345/export', {
            'Accept': 'application/zip',
            'X-UW-Act-as': 'bill'})

    @mock.patch('uw_catalyst.survey.get_resource')
    def test_survey_results(self, mock_fn):
        person = PWS().get_person_by_netid('bill')

        response = get_survey_results(12345, person)

        mock_fn.assert_called_with('/rest/webq/v2/survey/12345/responses', {
            'Accept': 'application/vnd.ms-excel',
            'X-UW-Act-as': 'bill'})

    @mock.patch('uw_catalyst.survey.get_resource')
    def test_survey_code_translation(self, mock_fn):
        person = PWS().get_person_by_netid('bill')

        response = get_survey_code_translation(12345, person)

        mock_fn.assert_called_with(
            '/rest/webq/v2/survey/12345/code-translation', {
                'Accept': 'text/csv',
                'X-UW-Act-as': 'bill'})



# Copyright 2021 UW-IT, University of Washington
# SPDX-License-Identifier: Apache-2.0

"""
Contains Catalyst DAO implementations.
"""
from restclients_core.dao import DAO
from os.path import abspath, dirname
from datetime import datetime
import hashlib
import pytz
import os


class Catalyst_DAO(DAO):
    def service_name(self):
        return "catalyst"

    def service_mock_paths(self):
        return [abspath(os.path.join(dirname(__file__), "resources"))]

    def _sol_auth_url(self, url):
        return url.replace("/rest/", "/js_rest/", 1)

    def _load_resource(self, method, url, headers, body):
        if self.get_service_setting("SOL_AUTH_PRIVATE_KEY") is not None:
            url = self._sol_auth_url(url)

        return super(Catalyst_DAO, self)._load_resource(
            method, url, headers, body)

    def _custom_headers(self, method, url, headers, body):
        private_key = self.get_service_setting("SOL_AUTH_PRIVATE_KEY")
        if private_key is not None:
            url = self._sol_auth_url(url)
            now_with_tz = datetime.now(pytz.utc).strftime(
                "%a %b %d %H:%M:%S %Z %Y")
            header_base = "%s\nGET\n%s\n%s\n" % (private_key, url, now_with_tz)
            hashed = hashlib.sha1(header_base.encode("utf-8")).hexdigest()
            public_key = self.get_service_setting("SOL_AUTH_PUBLIC_KEY")

            return {"Authorization": "SolAuth %s:%s" % (public_key, hashed),
                    "Date": now_with_tz}



# Copyright 2021 UW-IT, University of Washington
# SPDX-License-Identifier: Apache-2.0

"""
Contains custom exceptions used by the catalyst rest client.
"""


class InvalidGradebookID(Exception):
    """Exception for invalid gradebook id."""
    pass


class InvalidSurveyID(Exception):
    """Exception for invalid survey id."""
    pass



# Copyright 2021 UW-IT, University of Washington
# SPDX-License-Identifier: Apache-2.0

"""
Interface for interacting with Catalyst GradeBook
"""

from uw_catalyst.models import GradebookParticipant
from uw_catalyst.exceptions import InvalidGradebookID
from uw_catalyst import get_resource, get_json_resource, encode_section_label
import re


def get_gradebook_export(gradebook_id, person=None):
    """
    Returns a gradebook exported as an .xls file
    """
    if not valid_gradebook_id(gradebook_id):
        raise InvalidGradebookID(gradebook_id)

    url = "/rest/gradebook/v1/book/{}/export".format(gradebook_id)
    headers = {"Accept": "application/vnd.ms-excel"}

    if person is not None:
        headers["X-UW-Act-as"] = person.uwnetid

    return get_resource(url, headers)


def get_participants_for_gradebook(gradebook_id, person=None):
    """
    Returns a list of gradebook participants for the passed gradebook_id and
    person.
    """
    if not valid_gradebook_id(gradebook_id):
        raise InvalidGradebookID(gradebook_id)

    url = "/rest/gradebook/v1/book/{}/participants".format(gradebook_id)
    headers = {}

    if person is not None:
        headers["X-UW-Act-as"] = person.uwnetid

    data = get_json_resource(url, headers)

    participants = []
    for pt in data["participants"]:
        participants.append(_participant_from_json(pt))

    return participants


def get_participants_for_section(section, person=None):
    """
    Returns a list of gradebook participants for the passed section and person.
    """
    section_label = encode_section_label(section.section_label())
    url = "/rest/gradebook/v1/section/{}/participants".format(section_label)
    headers = {}

    if person is not None:
        headers["X-UW-Act-as"] = person.uwnetid

    data = get_json_resource(url, headers)

    participants = []
    for pt in data["participants"]:
        participants.append(_participant_from_json(pt))

    return participants


def valid_gradebook_id(gradebook_id):
    if re.match(r"^[1-9][0-9]{,9}$", str(gradebook_id)):
        return True
    return False


def _participant_from_json(data):
    participant = GradebookParticipant()
    participant.participant_id = data["participant_id"]
    participant.class_grade = data["class_grade"]
    participant.notes = data["notes"]
    participant.person_id = data["person_id"]
    return participant
